
#include <iostream>

#include "ijkgrid.txx"

using namespace std;

using namespace IJK;

typedef u_int AXIS_SIZE_TYPE;

typedef GRID_NEIGHBORS<u_int, AXIS_SIZE_TYPE, u_int, int, u_int> TGRID;

template <typename DTYPE, typename CTYPE>
void printcoord(DTYPE dimension, CTYPE * coord)
{
  cerr << "(";
  for (DTYPE d = 0; d < dimension; d++) {
    cerr << coord[d];
    if (d+1 < dimension) { cerr << ","; }
  }
  cerr << ")";
}

template <typename CTYPE>
void printcoord3D(CTYPE * coord)
{
  printcoord(3, coord);
}


int main() {

  int DIM3 = 3;
  AXIS_SIZE_TYPE axis_size[] = { 5, 6, 7 };
  int coord[] = { 1, 2, 3 };

  try {

  TGRID grid(DIM3, axis_size);

  int num_cube_vertices = grid.NumCubeVertices();
  cerr << "Num cube vertices = " << num_cube_vertices << endl;
  cerr << "Axis increment: ";
  for (int d = 0; d < DIM3; d++) {
    cerr << " " << grid.AxisIncrement(d);
  }
  cerr << endl;
  cerr << "Cube vertex increment: ";
  for (int i = 0; i < num_cube_vertices; i++) {
    cerr << " " << grid.CubeVertexIncrement(i);
  }
  cerr << endl;
  cerr << "Unit cube vertex coordinates: ";
  for (int i = 0; i < num_cube_vertices; i++) {
    cerr << " (";
    for (int d = 0; d < DIM3; d++) {
      cerr << grid.UnitCubeCoord(i,d);
      if (d+1 < DIM3) { cerr << ","; };
    }
    cerr << ")";
  }
  cerr << endl;

  cerr << "Num vertex neighbors (cube) = " << grid.NumVertexNeighborsC() << endl;
  int iv0 = grid.ComputeVertexIndex(coord);
  cerr << "Vertex " << iv0 << " Coordinates: ";
  printcoord3D(coord);
  cerr << endl;
  cerr << "Neighbors:" << endl;
  for (int k = 0; k < grid.NumVertexNeighborsC(); k++) {
    int iv = grid.VertexNeighborC(iv0, k);
    grid.ComputeCoord(iv, coord);
    cerr << "  " << iv << " ";
    printcoord3D(coord);
    cerr << endl;
  }
  cerr << endl;

  cerr << "Num vertex neighbors (edge) = " << grid.NumVertexNeighborsE() << endl;
  iv0 = grid.ComputeVertexIndex(coord);
  cerr << "Vertex " << iv0 << " Coordinates: ";
  printcoord3D(coord);
  cerr << endl;
  cerr << "Neighbors:" << endl;
  for (int k = 0; k < grid.NumVertexNeighborsE(); k++) {
    int iv = grid.VertexNeighborE(iv0, k);
    grid.ComputeCoord(iv, coord);
    cerr << "  " << iv << " ";
    printcoord3D(coord);
    cerr << endl;
  }
  cerr << endl;

  cerr << "Num facet neighbors = " << grid.NumFacetNeighborsC() << endl;
  iv0 = grid.ComputeVertexIndex(coord);
  cerr << "Vertex " << iv0 << " Coordinates: ";
  printcoord3D(coord);
  cerr << endl;
  for (int orth_dir = 0; orth_dir < DIM3; orth_dir++) {
    cerr << "Orthogonal direction " << orth_dir
	 << ".  Neighbors:" << endl;
    for (int k = 0; k < grid.NumFacetNeighborsC(); k++) {
      int iv = grid.FacetNeighborC(iv0, orth_dir, k);
      grid.ComputeCoord(iv, coord);
      cerr << "  " << iv << " ";
      printcoord3D(coord);
      cerr << endl;
    }
  }
  cerr << endl;

  cerr << "Num edge neighbors = " << grid.NumEdgeNeighborsF2() << endl;
  iv0 = grid.ComputeVertexIndex(coord);
  cerr << "Vertex " << iv0 << " Coordinates: ";
  printcoord3D(coord);
  cerr << endl;
  for (int dir = 0; dir < DIM3; dir++) {
    cerr << "Direction " << dir << ".  Neighbors:" << endl;
    for (int k = 0; k < grid.NumEdgeNeighborsF2(); k++) {
      int iv = grid.EdgeNeighborF2(iv0, dir, k);
      grid.ComputeCoord(iv, coord);
      cerr << "  " << iv << " ";
      printcoord3D(coord);
      cerr << endl;
    }
  }
  cerr << endl;

  }
  catch (IJK::PROCEDURE_ERROR & error) {
    if (error.NumMessages() == 0) {
      cerr << "Unknown error." << endl;
    }
    else { error.Print(cerr); }
    cerr << "Exiting." << endl;
    exit(20);
  }

}
